@@ -19,5 +19,7 @@
 bool client_proto(SBuf *sbuf, SBufEvent evtype, struct MBuf *pkt)  _MUSTCHECK;
 bool set_pool(PgSocket *client, const char *dbname, const char *username, const char *password, bool takeover) _MUSTCHECK;
 bool handle_auth_query_response(PgSocket *client, PktHdr *pkt);
-
+bool check_db_connection_count(PgSocket *client);
+bool check_user_connection_count(PgSocket *client);
+bool sending_auth_query(PgSocket *client);
 PgDatabase *prepare_auth_database(PgSocket *client) _MUSTCHECK;
