@@ -38,6 +38,8 @@ extern struct Slab *var_list_cache;
 extern struct Slab *server_prepared_statement_cache;
 extern PgPreparedStatement *prepared_statements;
 
+extern unsigned long long int last_pgsocket_id;
+
 PgDatabase *find_peer(int peer_id);
 PgDatabase *find_database(const char *name);
 PgDatabase *find_or_register_database(PgSocket *connection, const char *name);
@@ -63,15 +65,17 @@ void disconnect_client_sqlstate(PgSocket *client, bool notify, const char *sqlst
 PgDatabase * add_peer(const char *name, int peer_id) _MUSTCHECK;
 PgDatabase * add_database(const char *name) _MUSTCHECK;
 PgDatabase *register_auto_database(const char *name);
-PgGlobalUser * add_global_user(const char *name, const char *passwd) _MUSTCHECK;
-PgCredentials *add_global_credentials(const char *name, const char *passwd) _MUSTCHECK;
 PgCredentials * add_dynamic_credentials(PgDatabase *db, const char *name, const char *passwd) _MUSTCHECK;
 PgCredentials * force_user_credentials(PgDatabase *db, const char *username, const char *passwd) _MUSTCHECK;
 bool add_outstanding_request(PgSocket *client, char type, ResponseAction action) _MUSTCHECK;
-bool pop_outstanding_request(PgSocket *client, char *types, bool *skip);
-bool clear_outstanding_requests_until(PgSocket *server, char *types) _MUSTCHECK;
+bool pop_outstanding_request(PgSocket *client, const char types[], bool *skip);
+bool clear_outstanding_requests_until(PgSocket *server, const char types[]) _MUSTCHECK;
 bool queue_fake_response(PgSocket *client, char request_type) _MUSTCHECK;
 
+PgGlobalUser * update_global_user_passwd(PgGlobalUser *user, const char *passwd) _MUSTCHECK;
+PgGlobalUser * find_or_add_new_global_user(const char *name, const char *passwd) _MUSTCHECK;
+PgCredentials * find_or_add_new_global_credentials(const char *name, const char *passwd) _MUSTCHECK;
+
 PgCredentials * add_pam_credentials(const char *name, const char *passwd) _MUSTCHECK;
 
 void accept_cancel_request(PgSocket *req);
