@@ -61,7 +61,7 @@
 #define setsid() getpid()
 #define setgid(x) (-1)
 #define setuid(x) (-1)
-#define fork() (-1)
+#define fork() (errno = ENOSYS, -1)
 #define geteuid() getuid()
 static inline int setgroups(int ngroups, const gid_t *gidsets) { errno = EINVAL; return -1; }
 #define chown(f, u, g) (-1)
@@ -157,11 +157,4 @@ static inline struct group *getgrgid(gid_t gid) { return NULL; }
 
 #endif
 
-#define PRIdZ	"Id"
-#define PRIiZ	"Ii"
-#define PRIoZ	"Io"
-#define PRIuZ	"Iu"
-#define PRIxZ	"Ix"
-#define PRIXZ	"IX"
-
 #endif
