@@ -176,4 +176,8 @@ int tls_asn1_parse_time(struct tls *ctx, const ASN1_TIME *asn1time, time_t *dst)
 
 int asn1_time_parse(const char *, size_t, struct tm *, int);
 
+struct tls_keypair * tls_keypair_new(void);
+int tls_keypair_set_cert_file(struct tls_keypair *keypair, const char *cert_file);
+bool tls_keypair_list_equal(struct tls_keypair *tkp1, struct tls_keypair *tkp2);
+
 #endif /* HEADER_TLS_INTERNAL_H */
