@@ -535,7 +535,7 @@ bool set_pool(PgSocket *client, const char *dbname, const char *username, const
 
 		if (!check_user_connection_count(client))
 			return false;
-	} else if (client->client_auth_type == AUTH_TYPE_PAM) {
+	} else if (cf_auth_type == AUTH_TYPE_PAM) {
 		if (client->db->auth_user_credentials) {
 			slog_error(client, "PAM can't be used together with database authentication");
 			disconnect_client(client, true, "bouncer config error");
@@ -563,6 +563,7 @@ bool set_pool(PgSocket *client, const char *dbname, const char *username, const
 			return false;
 
 		if (!client->login_user_credentials || client->login_user_credentials->dynamic_passwd) {
+			PgGlobalUser *global_user;
 			/*
 			 * If the login user specified by the client
 			 * does not exist or if it has no entry in auth_file,
@@ -600,8 +601,22 @@ bool set_pool(PgSocket *client, const char *dbname, const char *username, const
 
 			slog_info(client, "no such user: %s", username);
 			client->login_user_credentials = calloc(1, sizeof(*client->login_user_credentials));
+
+			/*
+			 * For users that we are already tracking, we want to
+			 * track this correctly as a connection count. But for
+			 * users that we don't know about at all, we don't want
+			 * to create a new global user. That's why we use
+			 * find_global_user instead of
+			 * find_or_add_new_global_user.
+			 */
+			global_user = find_global_user(username);
+			if (global_user)
+				client->login_user_credentials->global_user = global_user;
+
 			if (!check_db_connection_count(client))
 				return false;
+
 			client->login_user_credentials->mock_auth = true;
 			safe_strcpy(client->login_user_credentials->name, username, sizeof(client->login_user_credentials->name));
 			if (!check_user_connection_count(client)) {
@@ -1018,6 +1033,7 @@ static bool decide_startup_pool(PgSocket *client, PktHdr *pkt)
 		if (!res) {
 			pktbuf_free(buf);
 			disconnect_client(client, false, "unable to send protocol negotiation packet");
+			return false;
 		}
 	}
 
