@@ -755,10 +755,6 @@ static bool parse_line(struct HBA *hba, struct Ident *ident, struct TokParser *t
 		rule->rule_method = AUTH_TYPE_PEER;
 	} else if (eat_kw(tp, "cert")) {
 		rule->rule_method = AUTH_TYPE_CERT;
-#ifdef HAVE_PAM
-	} else if (eat_kw(tp, "pam")) {
-		rule->rule_method = AUTH_TYPE_PAM;
-#endif
 	} else if (eat_kw(tp, "scram-sha-256")) {
 		rule->rule_method = AUTH_TYPE_SCRAM_SHA_256;
 	} else {
