@@ -1235,6 +1235,23 @@ static bool setup_tls(struct tls_config *conf, const char *pfx, int sslmode,
 	return true;
 }
 
+static bool tls_change_requires_reconnect(struct tls_config *new_server_connect_conf)
+{
+	if (server_connect_sslmode != cf_server_tls_sslmode) {
+		log_noise("new server_tls_sslmode detected");
+		return true;
+	} else if (server_connect_conf == NULL) {
+		log_noise("no existing server tls config detected");
+		return true;
+	} else if (tls_config_equal(new_server_connect_conf, server_connect_conf)) {
+		log_noise("no server tls config change detected");
+		return false;
+	} else {
+		log_noise("server tls config change detected");
+		return true;
+	}
+}
+
 bool sbuf_tls_setup(void)
 {
 	int err;
@@ -1255,7 +1272,7 @@ bool sbuf_tls_setup(void)
 			return false;
 		}
 	}
-	if (cf_auth_type == AUTH_CERT) {
+	if (cf_auth_type == AUTH_TYPE_CERT) {
 		if (cf_client_tls_sslmode != SSLMODE_VERIFY_FULL) {
 			log_error("auth_type=cert requires client_tls_sslmode=SSLMODE_VERIFY_FULL");
 			return false;
@@ -1319,11 +1336,8 @@ bool sbuf_tls_setup(void)
 	 * old TLS settings, possibly less secure, could be used for old
 	 * connections indefinitely. If TLS is disabled, and it was disabled before
 	 * as well then recycling connections is not necessary, since we know none
-	 * of the settings have changed. In all other cases we recycle the
-	 * connections to be on the safe side, even though it's possible nothing
-	 * has changed.
-	 */
-	if (server_connect_conf || new_server_connect_conf) {
+	 * of the settings have changed. */
+	if ((server_connect_conf || new_server_connect_conf) && tls_change_requires_reconnect(new_server_connect_conf)) {
 		struct List *item;
 		PgPool *pool;
 		statlist_for_each(item, &pool_list) {
