@@ -446,7 +446,9 @@ def test_user_client_count_db_connect_fail(pg, bouncer) -> None:
     assert user["current_client_connections"] == 0
 
     connect_args = {"dbname": test_dbname, "user": test_user}
-    with pytest.raises(psycopg.OperationalError):
+    with pytest.raises(
+        psycopg.OperationalError, match="pg_hba.conf rejects connection"
+    ):
         _ = bouncer.conn(**connect_args)
 
     users = bouncer.admin("SHOW USERS", row_factory=dict_row)
@@ -454,6 +456,41 @@ def test_user_client_count_db_connect_fail(pg, bouncer) -> None:
     assert user["current_client_connections"] == 0
 
 
+def test_user_client_count_db_connect_fail_2(bouncer) -> None:
+    test_user = "pswcheck_not_in_auth_file"
+    test_dbname = "user_passthrough"
+
+    users = bouncer.admin("SHOW USERS", row_factory=dict_row)
+    user = next(user for user in users if user["name"] == test_user)
+    assert user["current_client_connections"] == 0
+
+    connect_args = {"dbname": test_dbname, "user": test_user}
+    with pytest.raises(psycopg.OperationalError, match="authentication failed"):
+        _ = bouncer.conn(**connect_args)
+
+    users = bouncer.admin("SHOW USERS", row_factory=dict_row)
+    user = next(user for user in users if user["name"] == test_user)
+    assert user["current_client_connections"] == 0
+
+
+def test_user_client_count_db_connect_fail_3(bouncer) -> None:
+    test_user = "non_existent_user"
+    test_dbname = "user_passthrough"
+
+    users = bouncer.admin("SHOW USERS", row_factory=dict_row)
+    assert len([user for user in users if user["name"] == test_user]) == 0
+
+    connect_args = {"dbname": test_dbname, "user": test_user}
+    with pytest.raises(psycopg.OperationalError, match="authentication failed"):
+        _ = bouncer.conn(**connect_args)
+
+    users = bouncer.admin("SHOW USERS", row_factory=dict_row)
+    # We don't expect non-existent users which cannot authentiticate to show up
+    # in the stats at all. This would just pollute the stats output with people
+    # making typos or attackers trying random user accounts.
+    assert len([user for user in users if user["name"] == test_user]) == 0
+
+
 def test_min_pool_size_with_lower_max_user_connections(bouncer):
     # The p0x in test.init has min_pool_size set to 5. This should make
     # the PgBouncer try to create a pool for maxedout2 user of size 5 after a
